##############################################################################
#
# Copyright (c) 2001, 2006 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" FSSTXMethod: Filesystem methodish Structured Text document.
"""

from AccessControl.class_init import InitializeClass
from AccessControl.SecurityInfo import ClassSecurityInfo
from App.special_dtml import DTMLFile
from DocumentTemplate.DT_HTML import HTML as DTML_HTML
from OFS.DTMLDocument import DTMLDocument
from Products.PageTemplates.ZopePageTemplate import ZopePageTemplate
from zope.structuredtext import stx2html

from .DirectoryView import registerFileExtension
from .DirectoryView import registerMetaType
from .FSObject import FSObject
from .permissions import FTPAccess
from .permissions import View
from .permissions import ViewManagementScreens
from .utils import _checkConditionalGET
from .utils import _dtmldir
from .utils import _setCacheHeaders
from .utils import _ViewEmulator


_STX_TEMPLATE = 'ZPT'  # or 'DTML'

_DEFAULT_TEMPLATE_DTML = """\
<dtml-var standard_html_header>
<dtml-var cooked>
<dtml-var standard_html_footer>"""

_CUSTOMIZED_TEMPLATE_DTML = """\
<dtml-var standard_html_header>
<dtml-var stx fmt="structured-text">
<dtml-var standard_html_footer>"""

_DEFAULT_TEMPLATE_ZPT = """\
<html metal:use-macro="context/main_template/macros/master">
<body>

<metal:block metal:fill-slot="body"
><div tal:replace="structure options/cooked">
COOKED TEXT HERE
</div>
</metal:block>

</body>
</html>
"""

_CUSTOMIZED_TEMPLATE_ZPT = """\
<html metal:use-macro="context/main_template/macros/master">
<body>

<metal:block metal:fill-slot="body"
><div tal:define="std modules/Products/PythonScripts/standard;
                  stx nocall:std/structured_text;"
      tal:replace="structure python:stx(template.stx)">
COOKED TEXT HERE
</div>
</metal:block>

</body>
</html>
"""


class FSSTXMethod(FSObject):
    """ A chunk of StructuredText, rendered as a skin method of a CMF site.
    """
    meta_type = 'Filesystem STX Method'
    _owner = None  # unowned

    manage_options = ({'label': 'Customize', 'action': 'manage_main'},
                      {'label': 'View', 'action': ''})

    security = ClassSecurityInfo()
    security.declareObjectProtected(View)

    security.declareProtected(ViewManagementScreens, 'manage_main')
    manage_main = DTMLFile('custstx', _dtmldir)

    #
    #   FSObject interface
    #
    def _createZODBClone(self):
        """
            Create a ZODB (editable) equivalent of this object.
        """
        if _STX_TEMPLATE == 'DTML':
            target = DTMLDocument(_CUSTOMIZED_TEMPLATE_DTML,
                                  __name__=self.getId())
        elif _STX_TEMPLATE == 'ZPT':
            target = ZopePageTemplate(self.getId(), _CUSTOMIZED_TEMPLATE_ZPT)

        target._setProperty('stx', self.raw, 'text')
        return target

    def _readFile(self, reparse):
        """Read the data from the filesystem.
        """
        file = open(self._filepath)  # not 'rb', as this is a text file!
        try:
            data = file.read()
        finally:
            file.close()
        self.raw = data

        if reparse:
            self.cook()

    #
    #   "Wesleyan" interface (we need to be "methodish").
    #
    class _func_code:
        pass

    __code__ = _func_code()
    __code__.co_varnames = ()
    __code__.co_argcount = 0

    __defaults__ = ()

    index_html = None   # No accidental acquisition

    default_content_type = 'text/html'

    def cook(self):
        if not hasattr(self, '_v_cooked'):
            self._v_cooked = stx2html(self.raw, level=1, header=0)
        return self._v_cooked

    _default_DTML_template = DTML_HTML(_DEFAULT_TEMPLATE_DTML)
    _default_ZPT_template = ZopePageTemplate('stxmethod_view',
                                             _DEFAULT_TEMPLATE_ZPT,
                                             'text/html')

    def __call__(self, REQUEST={}, RESPONSE=None, **kw):
        """ Return our rendered StructuredText.
        """
        self._updateFromFS()

        if RESPONSE is not None:
            RESPONSE.setHeader('Content-Type', 'text/html')

        view = _ViewEmulator(self.getId()).__of__(self)
        _setCacheHeaders(view, extra_context={})

        if _checkConditionalGET(view, extra_context={}):
            return ''

        return self._render(REQUEST, RESPONSE, **kw)

    @security.private
    def modified(self):
        return self.getModTime()

    @security.private
    def _render(self, REQUEST={}, RESPONSE=None, **kw):
        """ Find the appropriate rendering template and use it to render us.
        """
        if _STX_TEMPLATE == 'DTML':
            default_template = self._default_DTML_template
        elif _STX_TEMPLATE == 'ZPT':
            default_template = self._default_ZPT_template
        else:
            raise TypeError('Invalid STX template: %s' % _STX_TEMPLATE)

        template = getattr(self, 'stxmethod_view', default_template)

        if getattr(template, 'isDocTemp', 0):
            posargs = (self, REQUEST)
        else:
            posargs = ()

        kwargs = {'cooked': self.cook()}
        return template(*posargs, **kwargs)

    @security.protected(FTPAccess)
    def manage_FTPget(self):
        """ Fetch our source for delivery via FTP.
        """
        return self.raw

    @security.protected(ViewManagementScreens)
    def PrincipiaSearchSource(self):
        """ Fetch our source for indexing in a catalog.
        """
        return self.raw

    @security.protected(ViewManagementScreens)
    def document_src(self):
        """ Fetch our source for rendering in the ZMI.
        """
        return self.raw


InitializeClass(FSSTXMethod)

registerFileExtension('stx', FSSTXMethod)
registerMetaType('STX Method', FSSTXMethod)



##############################################################################
#
# Copyright (c) 2005 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Cookie crumbler xml adapters and setup handlers.
"""

from zope.component import adapts
from zope.component import getSiteManager

from Products.GenericSetup.interfaces import ISetupEnviron
from Products.GenericSetup.utils import PropertyManagerHelpers
from Products.GenericSetup.utils import XMLAdapterBase
from Products.GenericSetup.utils import exportObjects
from Products.GenericSetup.utils import importObjects

from ..interfaces import ICookieCrumbler


class CookieCrumblerXMLAdapter(XMLAdapterBase, PropertyManagerHelpers):

    """XML im- and exporter for CookieCrumbler.
    """

    adapts(ICookieCrumbler, ISetupEnviron)

    _LOGGER_ID = 'cookies'

    name = 'cookieauth'

    def _exportNode(self):
        """Export the object as a DOM node.
        """
        node = self._getObjectNode('object')
        node.appendChild(self._extractProperties())

        self._logger.info('Cookie crumbler exported.')
        return node

    def _importNode(self, node):
        """Import the object from the DOM node.
        """
        if self.environ.shouldPurge():
            self._purgeProperties()

        self._migrateProperties(node)
        self._initProperties(node)

        self._logger.info('Cookie crumbler imported.')

    def _migrateProperties(self, node):
        # BBB: for CMF 2.2 settings
        for child in node.childNodes:
            if child.nodeName != 'property':
                continue
            if child.getAttribute('name') not in ('auto_login_page',
                                                  'unauth_page',
                                                  'logout_page'):
                continue
            node.removeChild(child)
            child.unlink()


def importCookieCrumbler(context):
    """Import cookie crumbler settings from an XML file.
    """
    sm = getSiteManager(context.getSite())
    tool = sm.queryUtility(ICookieCrumbler)
    if tool is None:
        logger = context.getLogger('cookies')
        logger.debug('Nothing to import.')
        return

    importObjects(tool, '', context)


def exportCookieCrumbler(context):
    """Export cookie crumbler settings as an XML file.
    """
    sm = getSiteManager(context.getSite())
    tool = sm.queryUtility(ICookieCrumbler)
    if tool is None:
        logger = context.getLogger('cookies')
        logger.debug('Nothing to export.')
        return

    exportObjects(tool, '', context)



from io import StringIO

from zExceptions import NotFound
from zope.event import notify
from zope.interface import alsoProvides
from zope.publisher.interfaces.browser import IBrowserRequest
from ZPublisher.HTTPRequest import HTTPRequest
from ZPublisher.HTTPResponse import HTTPResponse
from ZPublisher.pubevents import PubAfterTraversal

from ..interfaces import IPublishableThroughAcquisition
from ..testing import FunctionalZCMLLayer
from .base.dummy import DummyContent
from .base.testcase import RequestTest


class TestExplicitAcquisition(RequestTest):
    layer = FunctionalZCMLLayer

    def setUp(self):
        super().setUp()
        environment = {
            "URL": "",
            "PARENTS": [self.app],
            "REQUEST_METHOD": "GET",
            "SERVER_PORT": "80",
            "REQUEST_METHOD": "GET",
            "steps": [],
            "SERVER_NAME": "localhost",
            "_hacked_path": 0,
        }
        self.req = req = HTTPRequest(StringIO(), environment, HTTPResponse())
        req.other.update(environment)
        alsoProvides(req, IBrowserRequest)

        self.app.snuk = DummyContent(id="snuk")
        self.app.foo = DummyContent(id="foo")
        self.app.foo.bar = DummyContent(id="bar")

    def test_notfound_when_acquired(self):
        self.assertFalse(IPublishableThroughAcquisition.providedBy(self.req))
        self.req.traverse("foo/foo/bar/snuk/foo")
        with self.assertRaises(NotFound):
            notify(PubAfterTraversal(self.req))

    def test_allow_for_item(self):
        self.assertFalse(IPublishableThroughAcquisition.providedBy(self.req))
        alsoProvides(self.app.snuk, IPublishableThroughAcquisition)
        self.req.traverse("foo/foo/bar/snuk/foo")
        notify(PubAfterTraversal(self.req))

    def test_allow_for_request(self):
        self.assertFalse(IPublishableThroughAcquisition.providedBy(self.req))
        # not allowed
        self.req.traverse("foo/foo/bar/snuk/foo")
        with self.assertRaises(NotFound):
            notify(PubAfterTraversal(self.req))
        # allowed
        alsoProvides(self.req, IPublishableThroughAcquisition)
        notify(PubAfterTraversal(self.req))



from zope.interface import Interface


class IPublishableThroughAcquisition(Interface):
    """Marker interface that needs to be provided by content that is
    publishable through acquisition.
    """


class IShouldAllowAcquiredItemPublication(Interface):
    """Should we allow an (acquired) item?"""



import os

from zExceptions import NotFound
from zope.component import adapter
from ZPublisher.interfaces import IPubAfterTraversal

from Products.CMFCore.interfaces import IContentish
from Products.CMFCore.interfaces import IPublishableThroughAcquisition
from Products.CMFCore.interfaces import IShouldAllowAcquiredItemPublication


PTA_ENV_KEY = "PUBLISHING_EXPLICIT_ACQUISITION"
SKIP_PTA = os.environ.get(PTA_ENV_KEY, "true") == "false"


@adapter(IPubAfterTraversal)
def after_traversal_hook(event):
    if SKIP_PTA or IPublishableThroughAcquisition.providedBy(event.request):
        return
    context = event.request["PARENTS"][0]
    if IShouldAllowAcquiredItemPublication(context, None) is False:
        raise NotFound()


@adapter(IContentish)
def content_allowed(context):
    if IPublishableThroughAcquisition.providedBy(context):
        return True
    return context.aq_chain == context.aq_inner.aq_chain



##############################################################################
#
# Copyright (c) 2002 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" Unit tests for URLTool module.
"""

import unittest

from zope.component import getSiteManager
from zope.interface.verify import verifyClass
from zope.testing.cleanup import cleanUp

from ..interfaces import ISiteRoot
from .base.dummy import DummyContent
from .base.dummy import DummyFolder
from .base.dummy import DummySite


class URLToolTests(unittest.TestCase):

    def setUp(self):
        self.site = DummySite(id='foo')
        sm = getSiteManager()
        sm.registerUtility(self.site, ISiteRoot)

    def tearDown(self):
        cleanUp()

    def _makeOne(self, *args, **kw):
        from ..URLTool import URLTool

        url_tool = URLTool(*args, **kw)
        return url_tool.__of__(self.site)

    def test_interfaces(self):
        from ..interfaces import IActionProvider
        from ..interfaces import IURLTool
        from ..URLTool import URLTool

        verifyClass(IActionProvider, URLTool)
        verifyClass(IURLTool, URLTool)

    def test_portal_methods(self):
        url_tool = self._makeOne()
        self.assertEqual(url_tool(), 'http://www.foobar.com/bar/foo')
        self.assertEqual(url_tool.getPortalObject(), self.site)
        self.assertEqual(url_tool.getPortalPath(), '/bar/foo')

    def test_content_methods(self):
        url_tool = self._makeOne()
        self.site._setObject('folder', DummyFolder(id='buz'))
        self.site.folder._setObject('item', DummyContent(id='qux.html'))
        obj = self.site.folder.item
        self.assertEqual(url_tool.getRelativeContentPath(obj),
                         ('buz', 'qux.html'))
        self.assertEqual(url_tool.getRelativeContentURL(obj), 'buz/qux.html')
        self.assertEqual(url_tool.getRelativeUrl(obj), 'buz/qux.html')
